__all__ = ["MessageBoundMethods", "FileBoundMethods", "CallbackQueryBoundMethods"]
from .message import MessageBoundMethods
from .file import FileBoundMethods
from .callback_query import CallbackQueryBoundMethods
